@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2018-2023
+# © Copyright EnterpriseDB UK Limited 2018-2025
 #
 # This file is part of Barman.
 #
@@ -20,17 +20,14 @@ import json
 import logging
 from contextlib import closing
 
-from barman.clients.cloud_cli import (
-    GeneralErrorExit,
-    NetworkErrorExit,
-    OperationErrorExit,
-    create_argument_parser,
-)
+from barman.clients.cloud_cli import GeneralErrorExit, create_argument_parser
 from barman.cloud import CloudBackupCatalog, configure_logging
 from barman.cloud_providers import get_cloud_interface
 from barman.infofile import BackupInfo
 from barman.utils import force_str
 
+_logger = logging.getLogger(__name__)
+
 
 def main(args=None):
     """
@@ -46,20 +43,15 @@ def main(args=None):
         cloud_interface = get_cloud_interface(config)
 
         with closing(cloud_interface):
+            # Do connectivity test if requested
+            if config.test:
+                cloud_interface.verify_cloud_connectivity_and_bucket_existence()
+                raise SystemExit(0)
+
             catalog = CloudBackupCatalog(
                 cloud_interface=cloud_interface, server_name=config.server_name
             )
 
-            if not cloud_interface.test_connectivity():
-                raise NetworkErrorExit()
-            # If test is requested, just exit after connectivity test
-            elif config.test:
-                raise SystemExit(0)
-
-            if not cloud_interface.bucket_exists:
-                logging.error("Bucket %s does not exist", cloud_interface.bucket_name)
-                raise OperationErrorExit()
-
             backup_list = catalog.get_backup_list()
 
             # Output
@@ -103,8 +95,8 @@ def main(args=None):
                 )
 
     except Exception as exc:
-        logging.error("Barman cloud backup list exception: %s", force_str(exc))
-        logging.debug("Exception details:", exc_info=exc)
+        _logger.error("Barman cloud backup list exception: %s", force_str(exc))
+        _logger.debug("Exception details:", exc_info=exc)
         raise GeneralErrorExit()
 
 
